@@ -6,10 +6,12 @@
 import argparse
 import base64
 import dataclasses
+import datetime
 import json
 import os
 import re
 import shlex
+import shutil
 import subprocess
 import sys
 import tempfile
@@ -36,21 +38,13 @@ ExecStart=false
 """
 
 
-def sandbox(args: argparse.Namespace) -> list[str]:
-    return [
-        args.mkosi,
-        '--directory', os.fspath(args.meson_source_dir),
-        '--extra-search-path', os.fspath(args.meson_build_dir),
-        'sandbox',
-    ]  # fmt: skip
-
-
 @dataclasses.dataclass(frozen=True)
 class Summary:
     distribution: str
     release: str
     architecture: str
     builddir: Path
+    buildsubdir: Path
     environment: dict[str, str]
 
     @classmethod
@@ -73,6 +67,7 @@ class Summary:
             release=j['Images'][-1]['Release'],
             architecture=j['Images'][-1]['Architecture'],
             builddir=Path(j['Images'][-1]['BuildDirectory']),
+            buildsubdir=Path(j['Images'][-1]['BuildSubdirectory']),
             environment=j['Images'][-1]['Environment'],
         )
 
@@ -86,7 +81,7 @@ def process_coredumps(args: argparse.Namespace, journal_file: Path) -> bool:
         exclude_regex = None
 
     result = subprocess.run(
-        sandbox(args) + [
+        [
             'coredumpctl',
             '--file', journal_file,
             '--json=short',
@@ -109,7 +104,7 @@ def process_coredumps(args: argparse.Namespace, journal_file: Path) -> bool:
         return False
 
     subprocess.run(
-        sandbox(args) + [
+        [
             'coredumpctl',
             '--file', journal_file,
             '--no-pager',
@@ -151,7 +146,7 @@ def process_sanitizer_report(args: argparse.Namespace, journal_file: Path) -> bo
     find_comm = re.compile(r'^\[[.0-9 ]+?\]\s(.*?:)\s')
 
     with subprocess.Popen(
-        sandbox(args) + [
+        [
             'journalctl',
             '--output', 'short-monotonic',
             '--no-hostname',
@@ -245,7 +240,7 @@ def process_sanitizer_report(args: argparse.Namespace, journal_file: Path) -> bo
 
 def process_coverage(args: argparse.Namespace, summary: Summary, name: str, journal_file: Path) -> None:
     coverage = subprocess.run(
-        sandbox(args) + [
+        [
             'journalctl',
             '--file', journal_file,
             '--field=COVERAGE_TAR',
@@ -265,7 +260,7 @@ def process_coverage(args: argparse.Namespace, summary: Summary, name: str, jour
 
         with tempfile.TemporaryDirectory(prefix='coverage-') as tmp:
             subprocess.run(
-                sandbox(args) + [
+                [
                     'tar',
                     '--extract',
                     '--file', '-',
@@ -287,7 +282,7 @@ def process_coverage(args: argparse.Namespace, summary: Summary, name: str, jour
                 p.rename(dst)
 
             subprocess.run(
-                sandbox(args) + [
+                [
                     'find',
                     tmp,
                     '-name', '*.gcda',
@@ -299,23 +294,21 @@ def process_coverage(args: argparse.Namespace, summary: Summary, name: str, jour
             )  # fmt: skip
 
             subprocess.run(
-                sandbox(args)
-                + [
+                [
                     'rsync',
                     '--archive',
                     '--prune-empty-dirs',
                     '--include=*/',
                     '--include=*.gcno',
                     '--exclude=*',
-                    f'{os.fspath(args.meson_build_dir / summary.builddir)}/',
+                    f'{os.fspath(summary.builddir / summary.buildsubdir)}/',
                     os.fspath(Path(tmp) / 'work/build'),
                 ],
                 check=True,
             )
 
             subprocess.run(
-                sandbox(args)
-                + [
+                [
                     'lcov',
                     *(
                         [
@@ -336,11 +329,11 @@ def process_coverage(args: argparse.Namespace, summary: Summary, name: str, jour
                     '--quiet',
                 ],
                 check=True,
+                cwd=os.fspath(args.meson_source_dir),
             )  # fmt: skip
 
             subprocess.run(
-                sandbox(args)
-                + [
+                [
                     'lcov',
                     '--ignore-errors', 'inconsistent,inconsistent,format,corrupt,empty',
                     '--add-tracefile', output if output.exists() else initial,
@@ -349,6 +342,7 @@ def process_coverage(args: argparse.Namespace, summary: Summary, name: str, jour
                     '--quiet',
                 ],
                 check=True,
+                cwd=os.fspath(args.meson_source_dir),
             )  # fmt: skip
 
             Path(f'{output}.new').unlink()
@@ -370,6 +364,7 @@ def main() -> None:
     parser.add_argument('--exit-code', required=True, type=int)
     parser.add_argument('--coredump-exclude-regex', required=True)
     parser.add_argument('--sanitizer-exclude-regex', required=True)
+    parser.add_argument('--rtc', action=argparse.BooleanOptionalAction)
     parser.add_argument('mkosi_args', nargs='*')
     args = parser.parse_args()
 
@@ -400,7 +395,7 @@ def main() -> None:
     shell = bool(int(os.getenv('TEST_SHELL', '0')))
     summary = Summary.get(args)
 
-    if shell and not sys.stderr.isatty():
+    if shell and not sys.stdin.isatty():
         print(
             '--interactive must be passed to meson test to use TEST_SHELL=1',
             file=sys.stderr,
@@ -429,7 +424,7 @@ def main() -> None:
         dropin += textwrap.dedent(
             f"""
             [Service]
-            Environment=TEST_MATCH_SUBTEST={os.environ["TEST_MATCH_SUBTEST"]}
+            Environment=TEST_MATCH_SUBTEST={os.environ['TEST_MATCH_SUBTEST']}
             """
         )
 
@@ -437,14 +432,18 @@ def main() -> None:
         dropin += textwrap.dedent(
             f"""
             [Service]
-            Environment=TEST_MATCH_TESTCASE={os.environ["TEST_MATCH_TESTCASE"]}
+            Environment=TEST_MATCH_TESTCASE={os.environ['TEST_MATCH_TESTCASE']}
             """
         )
 
-    journal_file = (args.meson_build_dir / (f'test/journal/{name}.journal')).absolute()
+    if os.getenv('TEST_JOURNAL_USE_TMP', '0') == '1':
+        journal_file = Path(f'/tmp/systemd-integration-tests/journal/{name}.journal')
+    else:
+        journal_file = (args.meson_build_dir / f'test/journal/{name}.journal').absolute()
+
     journal_file.unlink(missing_ok=True)
 
-    if not sys.stderr.isatty():
+    if not sys.stdin.isatty():
         dropin += textwrap.dedent(
             """
             [Unit]
@@ -455,29 +454,59 @@ def main() -> None:
         dropin += textwrap.dedent(
             """
             [Unit]
-            Wants=multi-user.target
+            Wants=multi-user.target getty-pre.target
+            Before=getty-pre.target
+
+            [Service]
+            StandardInput=tty
+            StandardOutput=inherit
+            StandardError=inherit
+            TTYReset=yes
+            TTYVHangup=yes
+            IgnoreSIGPIPE=no
+            # bash ignores SIGTERM
+            KillSignal=SIGHUP
             """
         )
 
+    if sys.stdin.isatty():
+        dropin += textwrap.dedent(
+            """
+            [Service]
+            ExecStartPre=/usr/lib/systemd/tests/testdata/integration-test-setup.sh setup
+            ExecStopPost=/usr/lib/systemd/tests/testdata/integration-test-setup.sh finalize
+            StateDirectory=%N
+            """
+        )
+
+    if args.rtc:
+        if sys.version_info >= (3, 12):
+            now = datetime.datetime.now(datetime.UTC)
+        else:
+            now = datetime.datetime.utcnow()
+
+        rtc = datetime.datetime.strftime(now, r'%Y-%m-%dT%H:%M:%S')
+    else:
+        rtc = None
+
     cmd = [
         args.mkosi,
         '--directory', os.fspath(args.meson_source_dir),
-        '--output-dir', os.fspath(args.meson_build_dir / 'mkosi.output'),
-        '--extra-search-path', os.fspath(args.meson_build_dir),
         '--machine', name,
-        '--ephemeral',
+        '--ephemeral=yes',
         *(['--forward-journal', journal_file] if journal_file else []),
         *(
             [
                 '--credential', f'systemd.extra-unit.emergency-exit.service={shlex.quote(EMERGENCY_EXIT_SERVICE)}',  # noqa: E501
                 '--credential', f'systemd.unit-dropin.emergency.target={shlex.quote(EMERGENCY_EXIT_DROPIN)}',
             ]
-            if not sys.stderr.isatty()
+            if not sys.stdin.isatty()
             else []
         ),
         '--credential', f'systemd.unit-dropin.{args.unit}={shlex.quote(dropin)}',
         '--runtime-network=none',
         '--runtime-scratch=no',
+        *([f'--qemu-args=-rtc base={rtc}'] if rtc else []),
         *args.mkosi_args,
         '--firmware', args.firmware,
         *(['--kvm', 'no'] if int(os.getenv('TEST_NO_KVM', '0')) else []),
@@ -496,13 +525,13 @@ def main() -> None:
                         'systemd.crash_action=poweroff',
                         'loglevel=6',
                     ]
-                    if not sys.stderr.isatty()
+                    if not sys.stdin.isatty()
                     else []
                 ),
             ]
         ),
-        '--credential', f"journal.storage={'persistent' if sys.stderr.isatty() else args.storage}",
-        *(['--runtime-build-sources=no'] if not sys.stderr.isatty() else []),
+        '--credential', f"journal.storage={'persistent' if sys.stdin.isatty() else args.storage}",
+        *(['--runtime-build-sources=no', '--register=no'] if not sys.stdin.isatty() else []),
         'vm' if args.vm or os.getuid() != 0 or os.getenv('TEST_PREFER_QEMU', '0') == '1' else 'boot',
     ]  # fmt: skip
 
@@ -541,6 +570,10 @@ def main() -> None:
         and not sanitizer
     ):
         journal_file.unlink(missing_ok=True)
+    elif os.getenv('TEST_JOURNAL_USE_TMP', '0') == '1':
+        dst = args.meson_build_dir / f'test/journal/{name}.journal'
+        dst.parent.mkdir(parents=True, exist_ok=True)
+        shutil.move(journal_file, dst)
 
     if shell or (result.returncode in (args.exit_code, 77) and not coredumps and not sanitizer):
         exit(0 if shell or result.returncode == args.exit_code else 77)
@@ -559,7 +592,7 @@ def main() -> None:
 
     ops += [f'journalctl --file {journal_file} --no-hostname -o short-monotonic -u {args.unit} -p info']
 
-    print("Test failed, relevant logs can be viewed with: \n\n" f"{(' && '.join(ops))}\n", file=sys.stderr)
+    print(f'Test failed, relevant logs can be viewed with: \n\n{(" && ".join(ops))}\n', file=sys.stderr)
 
     # 0 also means we failed so translate that to a non-zero exit code to mark the test as failed.
     exit(result.returncode or 1)
